/////////////////////////////////////////////////////////////////////////////////
// player.gs
// included by game.gs
// Deals with player general actions and others
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
// Utils
/////////////////////////////////////////////////////////////////////////////////

// OUT: int(0/1)
// Tests if player should update this frame (syncronization).
func IsPlayerUpdate()
{
	return IsUpdate(PlayerGet(P_DELAY));
}

// IN: int(material)
// OUT: int(0/1)
// Tests if the material is found inside player's bounding box.
func IsMaterialInsidePlayer( mat )
{
	return PlayerGet(P_MATINSIDE) & (1<<mat);
}

// IN: int(material)
// OUT: int(0/1)
// Tests if the material is found just under player's bounding box.
func IsMaterialUnderPlayer( mat )
{
	return PlayerGet(P_MATUNDER) & (1<<mat);
}

// OUT: int(0/1)
// Tests if player's position is considered safe for respawning or dropping objects.
func IsPlayerSafe()
{
	matmask = (1<<MAT_AIR)|(1<<MAT_CLOUD)|(1<<MAT_WATER); // only those are allowed inside player's box
	return ( 0 == ( PlayerGet(P_MATINSIDE) & (~matmask) ) ); // no other materials are allowed
}

// OUT: int(0/1)
// Tests if player's position is considered stable for respawning or dropping objects.
func IsPlayerStable()
{
	matmask = (1<<MAT_BLOCK)|(1<<MAT_JUMPFIX)|(1<<MAT_JUMPPRO); // at least one material with hard density must be under the player
	return ( 0 != ( PlayerGet(P_MATUNDER) & matmask ) );
}

// IN: int(x1), int(y1), int(x2), int(y2)
// OUT: int(0/1)
// Tests if player touches a rectangle (bounding boxes intersect)
func PlayerTouchRect( x1, y1, x2, y2 )
{
	pw = PlayerGet(P_W)+4; // use a small horisontal boggus (-2/+2) so it can pick hard objects
	ph = PlayerGet(P_H);
	px1 = PlayerGet(P_X)-pw/2;
	py1 = PlayerGet(P_Y)-ph/2;
	px2 = px1+pw;
	py2 = py1+ph;
	if( px1>=x2 || px2<=x1 ) return 0;
	if( py1>=y2 || py2<=y1 ) return 0;
	return 1;
}

// IN: int(object index)
// OUT: int(0/1)
// Tests if player touches an object (bounding boxes intersect)
func PlayerTouchObject( idx )
{
	pw = PlayerGet(P_W)+4; // use a small horisontal boggus (-2/+2) so it can pick hard objects
	ph = PlayerGet(P_H);
	px1 = PlayerGet(P_X)-pw/2;
	py1 = PlayerGet(P_Y)-ph/2;
	px2 = px1+pw;
	py2 = py1+ph;
	ox1 = ObjGet(idx,O_X);
	oy1 = ObjGet(idx,O_Y);
	ox2 = ox1+ObjGet(idx,O_W);
	oy2 = oy1+ObjGet(idx,O_H);
	if( px1>=ox2 || px2<=ox1 ) return 0;
	if( py1>=oy2 || py2<=oy1 ) return 0;
	return 1;
}


/////////////////////////////////////////////////////////////////////////////////
// Entering states
/////////////////////////////////////////////////////////////////////////////////

func PlayerEnterIdle()
{
	PlayerSet(P_STATUS,STATUS_IDLE);
	PlayerSet(P_DIR,0);
	PlayerSet(P_POW,0);
	PlayerSet(P_FLIP,PlayerGet(P_FLIP)&FLIPY);
	PlayerSet(P_FRAME,0);
	PlayerSet(P_ANIM,2);
	PlayerSet(P_TILE,PlayerGet(P_TILEIDLE)+PlayerGet(P_EMOTION));
	PlayerSet(P_STUNLEVEL,0); // avoid resident stun :)
}

// IN: int(dir -1/1)
func PlayerEnterWalk( dir )
{
	PlayerSet(P_STATUS,STATUS_WALK);
	PlayerSet(P_DIR,dir);
	PlayerSet(P_POW,0);
	PlayerSet(P_FLIP,(PlayerGet(P_FLIP)&FLIPY)|(dir==-1));
	PlayerSet(P_FRAME,0);
	PlayerSet(P_ANIM,2);
	PlayerSet(P_TILE,PlayerGet(P_TILEWALK));
}

// IN: int(dir -1/0/1), int(jump power)
func PlayerEnterJump( dir, pow )
{
	PlayerSet(P_STATUS,STATUS_JUMP);
	PlayerSet(P_DIR,dir);
	PlayerSet(P_POW,pow);
	PlayerSet(P_FLIP,(PlayerGet(P_FLIP)&FLIPY)|(dir==-1));
	PlayerSet(P_FRAME,0);
	PlayerSet(P_ANIM,2);
	if(dir==0)
		PlayerSet(P_TILE,PlayerGet(P_TILEUP));
	else
		PlayerSet(P_TILE,PlayerGet(P_TILEJUMP));
}

func PlayerEnterFall()
{
	PlayerSet(P_STATUS,STATUS_FALL);
	PlayerSet(P_POW,1);
	PlayerSet(P_ANIM,2);
}

// IN: int(dir -1/0/1)
func PlayerEnterSpin( dir )
{
	PlayerEnterFall();
	PlayerSet(P_DIR,dir);
	PlayerSet(P_POW,0);
	PlayerSet(P_FLIP,(PlayerGet(P_FLIP)&FLIPY)|(dir==-1));
	PlayerSet(P_FRAME,1);
	PlayerSet(P_ANIM,2);
	PlayerSet(P_TILE,PlayerGet(P_TILEJUMP));
}

func PlayerEnterScripted()
{
	PlayerSet(P_STATUS,STATUS_SCRIPTED);
}

// IN: int(x), int(y)
func PlayerSetPos( x, y )
{
	PlayerSet(P_X,x);
	PlayerSet(P_Y,y);
}


/////////////////////////////////////////////////////////////////////////////////
// Animations
/////////////////////////////////////////////////////////////////////////////////

// IN: int(tile)
// Player plays all frames from the tile's animation.
func PlayerPlayAnim( tile )
{
	PlayerSet( P_STATUS, STATUS_SCRIPTED );
	PlayerSet( P_TILE, tile );
	PlayerSet( P_FRAME, 0 );
	PlayerSet( P_ANIM, 1 );
	idx = TileFind( tile );
	frames = TileGet( idx, TILE_FRAMES ) * PlayerGet( P_DELAY );
	WaitFrames(frames);
}

// IN: int(tile), tab(frames)
// Players plays all the frames specified in the table.
func PlayerPlayAnimFrames( tile, frames )
{
	PlayerSet( P_STATUS, STATUS_SCRIPTED );
	PlayerSet( P_TILE, tile );
	PlayerSet( P_ANIM, 0 );
	idx = TileFind( tile );
	delay = PlayerGet( P_DELAY );
	for(i=0;i<sizeof(frames);i++)
	{
		PlayerSet( P_FRAME, frames[i] );
		WaitFrames(delay);
	}
}

// Stun event requested by HandlerStun (latent).
func PlayerPlayStun()
{
	if( IsPlayerStable() ) // hit hard
	{
		DoRumble(20); // DoShake(20);
		SamplePlay(FX_STUN);
		PlayerPlayAnimFrames(14, {0,1,0,1,0,1,0,1,2,3,2,3,2,3,2,3,4,4,4,4} );
	}
	PlayerEnterIdle();
}

// Dead event requested by HandlerDead (latent).
func PlayerPlayDead()
{
	SamplePlay(FX_DEATH);
	PlayerPlayAnim(15);
	PlayerLoseLife();
}

// Dead in water event requested by HandlerDead (latent).
func PlayerPlayDeadWater()
{
	SamplePlay(FX_DEATH);
	PlayerSet( P_STATUS, STATUS_SCRIPTED );
	PlayerSet( P_TILE, PlayerGet( P_TILEUP ) );
	PlayerSet( P_FRAME, 1 );
	PlayerSet( P_ANIM, 0 );
	
	while(PlayerGet(P_MATCENTER)==MAT_WATER)
	{
		//PlayerSet( P_FRAME, PlayerGet(P_FRAME)+1 );
		PlayerSet(P_Y, PlayerGet(P_Y)-2);
		WaitFrames(3);
		stop;
	}
	
	PlayerLoseLife();
}

// Lose life event (latent).
func PlayerLoseLife()
{
	credit = PlayerGet(P_CREDITS)-1;
	PlayerSet(P_CREDITS, credit);
	if(credit==0)
	{
		OpenDialogMessage("YOU HAVE LOST\nALL YOUR LIVES!");
		GameCommand(CMD_START);
		return;
	}

	OpenDialogMessage("YOU HAVE LOST\nONE LIFE!");
	
	// respawn
	SamplePlay(FX_RESPAWN);
	PlayerSet(P_LIFE, 100);
	PlayerSet(P_X, PlayerGet(P_XSAFE));
	PlayerSet(P_Y, PlayerGet(P_YSAFE));
	PlayerEnterIdle();
	MusicPlay(GetLocationMusic(),0,3);
}

// IN: int(damage)
// Player hurt event (non-latent).
func PlayerHurt( damage )
{
	DoRumble(6); // DoShake(2);
	SamplePlay(FX_HURT);
	life = PlayerGet(P_LIFE) - damage;
	if(life<=0) life=0;
	PlayerSet(P_LIFE,life);
	PlayerSet(P_SAFE,0); // not safe
}

/////////////////////////////////////////////////////////////////////////////////
