/////////////////////////////////////////////////////////////////////////////////
// util.gs
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
// IN: int(frames)
// Waits a specified number of game cycles.
/////////////////////////////////////////////////////////////////////////////////
func WaitFrames( frames )
{
	for(i=0;i<frames;i++) stop;
}

func WaitFramesClearKeys( frames )
{
	for(i=0;i<frames;i++) { ClearKeys(); stop; }
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(seconds)
// Waits a specified number of seconds.
/////////////////////////////////////////////////////////////////////////////////
func WaitTime( seconds )
{
	frames = seconds*GameGet(G_FPS);
	for(i=0;i<frames;i++) stop;
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(key index [0..4])
// OUT: int(0/1)
// Tests if a key is pressed down.
/////////////////////////////////////////////////////////////////////////////////
func GetKey( key )
{
	return GameGet(G_KEYS) & (1<<key);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(key index [0..4])
// OUT: int(0/1)
// Tests if a key has just been pressed (hit).
/////////////////////////////////////////////////////////////////////////////////
func GetKeyHit( key )
{
	return GameGet(G_KEYSHIT) & (1<<key);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(key index [0..4]), int(0/1)
// Overwrites a key value.
// Can be used to simulate player's input.
/////////////////////////////////////////////////////////////////////////////////
func SetKey( key, val )
{
	keys = GameGet(G_KEYS);
	keys &= ~(1<<key);
	keys |= val<<key;
	GameSet(G_KEYS,keys);
}

/////////////////////////////////////////////////////////////////////////////////
// Clears key values.
// Can be after closing menus, to prevent them from reopening.
/////////////////////////////////////////////////////////////////////////////////
func ClearKeys()
{
	GameSet(G_KEYS,0);
	GameSet(G_KEYSHIT,0);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(delay)
// OUT: int(0/1)
// Test if time for object's update (usually delay is object's O_DELAY).
/////////////////////////////////////////////////////////////////////////////////
func IsUpdate( delay )
{
	if(delay==0) return true;
	return (GameFrame()%delay==0);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(0/1)
// Pauses or unpauses the game.
/////////////////////////////////////////////////////////////////////////////////
func GamePause( on )
{
	GameSet(G_PAUSE,on);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(object index)
// OUT: int(0/1)
// Tests if object is pickable.
/////////////////////////////////////////////////////////////////////////////////
func ObjIsPickup( idx )
{
	class = ObjGet(idx,O_CLASS);
	return ( class==CLASS_ITEM || class==CLASS_COIN || class==CLASS_FOOD || class==CLASS_LIFE );
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(object index)
// OUT: int(0/1)
// Tests if object is actionable.
/////////////////////////////////////////////////////////////////////////////////
func ObjIsAction( idx )
{
	class = ObjGet(idx,O_CLASS);
	return ( class==CLASS_ACTION );
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(object index)
// Object plays all frames from the current tile's animation.
/////////////////////////////////////////////////////////////////////////////////
func ObjPlayAnim( idx )
{
	ObjSet( idx, O_FRAME, 0 );
	ObjSet( idx, O_ANIM, 1 );
	tileidx = TileFind( ObjGet(idx, O_TILE) );
	frames = TileGet( tileidx, TILE_FRAMES ) * ObjGet( idx, O_DELAY );
	WaitFrames(frames);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(object index), tab(frames)
// Object plays all the frames specified in the table from the current tile.
/////////////////////////////////////////////////////////////////////////////////
func ObjPlayAnimFrames( idx, frames )
{
	ObjSet( idx, O_ANIM, 0 );
	delay = ObjGet( idx, O_DELAY );
	for(i=0;i<sizeof(frames);i++)
	{
		ObjSet( idx, O_FRAME, frames[i] );
		WaitFrames(delay);
	}
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(frames)
// Request rumble for a number of frames
/////////////////////////////////////////////////////////////////////////////////
func DoRumble( frames )
{
	if(frames>GameGet(G_RUMBLE)) GameSet(G_RUMBLE,frames);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: int(frames)
// Request shake for a number of frames
/////////////////////////////////////////////////////////////////////////////////
func DoShake( frames )
{
	if(frames>GameGet(G_SHAKE)) GameSet(G_SHAKE,frames);
}	

/////////////////////////////////////////////////////////////////////////////////
// IN: int(magnitudex), int(frequencyx), int(magnitudey), int(frequencyy)
// Set shake offsets using sine courves
/////////////////////////////////////////////////////////////////////////////////
func ApplyShake( magnitudex, magnitudey, frequencyx, frequencyy )
{
	sx = gs_cos( (float)gs_time() / (float)frequencyx ) * ((float)magnitudex + 0.1);
	sy = gs_sin( (float)gs_time() / (float)frequencyy ) * ((float)magnitudey + 0.1);
	GameSet(G_SHAKEX,(int)sx);
	GameSet(G_SHAKEY,(int)sy);
}

/////////////////////////////////////////////////////////////////////////////////
// IN: str(filename)
// Loads rooms names saved from the editor v1.2
// Each line in the file has the folowing format: ROOMX; ROOMY; ROOMNAME
/////////////////////////////////////////////////////////////////////////////////
func RoomsLoadNames( filename )
{
	f = gs_fileopen(filename,0);
	if(f==0) return false;
	while(true)
	{
		entry = "";
		eof = (gs_filereadtext(&entry,f)==0);
		if(eof) break;
		t = strexplode(entry,';');
		if(sizeof(t)!=3) continue;
		RoomSetName((int)t[0],(int)t[1],t[2]);
	}
	gs_fileclose(f);	
	return true;
}

/////////////////////////////////////////////////////////////////////////////////
